<?php
/**
 * @package    es-f
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2007-2009 Knut Kohl
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 * @since      File available since Release 0.0.1
 */

abstract class Translation {

  /**
   * Separator to split key into array
   *
   * @access public
   * @static
   */
  public static $NameSpaceSeparator = '::';

  /**
   * Load translations from XML files
   *
   * @param string $XMLURIs Files pattern
   */
  public static function LoadXMLFiles( $XMLURIs ) {
    foreach (glob($XMLURIs) as $XMLURI) self::LoadXMLFile($XMLURI);
  }

  /**
   * Load translations from XML file
   *
   * @param string $XMLURI
   */
  public static function LoadXMLFile( $XMLURI ) {
    $xml = new XML_Array_Translation;
    // make keys uppercase
    $xml->Key2Lower = FALSE;

    $trans = $xml->ParseXMLFile($XMLURI);

    if (!$trans) throw new Exception($xml->Error);

    foreach ($trans as $Namespace => $translations) {
      foreach ($translations as $key => $data) {
        $filter = strtolower($data['filter']);
        $str = $data['data'];

        if ($filter) {

          $isHTML = FALSE;

          switch (TRUE) {

            case ($filter == 'html'):
              // text contains html tags
              $isHTML = TRUE;
              break;

            case ($filter == 'nl2br'):
              $str = nl2br($str);
              // un-HTML
              $str = str_replace(self::$Masks[0], self::$Masks[1], $str);
              break;

            case ($filter == 'p'):
              $str = str_replace("\n\n", '</p><p>', $str);
              $str = '<p>' . nl2br($str) . '</p>';
              // un-HTML
              $str = str_replace(self::$Masks[0], self::$Masks[1], $str);
              break;

            case ($filter == 'file'):
              // include text file
              $str = file_exists($str)
                   ? file_get_contents($str)
                   : Messages::toStr('ERROR: Missing translation file ['.$str.']', Messages::ERROR);
              // file is by default interpreted as HTML!
              $isHTML = TRUE;
              break;

          }   // switch

          if (!$isHTML) {
            $str = htmlspecialchars($str, ENT_COMPAT, 'UTF-8');
            $str = str_replace('"', '&quot;', $str);
          }
          // re-HTML
          $str = str_replace(self::$Masks[1], self::$Masks[0], $str);
        }
        self::$Translation[$Namespace][$key] = $str;
      }
    }
  }

  /**
   * Define translations
   *
   * @param string $Namespace
   * @param array $translations Translations
   */
  public static function Define( $Namespace, $translations ) {

    foreach ($translations as $key => $str) {
      $key = strtoupper($key);
      // test for functions
      if (strpos($str, ':') !== FALSE) {
        // at least one collon...
        list($func, $str) = explode(':', $str, 2);
        // we possibly found a function
        $isHTML = FALSE;
        switch (TRUE) {
          case ($func == 'html'):
            // text contains html tags
            $isHTML = TRUE;
            break;
          case ($func == 'nl2br'):
            $str = nl2br($str);
            // un-HTML
            $str = str_replace(self::$Masks[0], self::$Masks[1], $str);
            break;
          case ($func == 'p'):
            $str = str_replace("\n\n",'</p><p>',$str);
            $str = '<p>'.nl2br($str).'</p>';
            // un-HTML
            $str = str_replace(self::$Masks[0], self::$Masks[1], $str);
            break;
          case ($func == 'file'):
            // include text file
            $str = file_exists($str)
                 ? file_get_contents($str)
                 : Messages::toStr('ERROR: Missing translation file ['.$str.']', Messages::ERROR);
            // file is by default interpreted as HTML!
            $isHTML = TRUE;
            break;
          case isset($func):
            // assume, we got an collon inside text
            $str = $func.':'.$str;
            $func = '';
            break;
        }

        if (!$isHTML) {
          $str = htmlspecialchars($str, ENT_COMPAT, 'UTF-8');
          $str = str_replace('"','&quot;',$str);
        }
        // re-HTML
        $str = str_replace(self::$Masks[1], self::$Masks[0], $str);
      }
      self::$Translation[strtoupper($Namespace)][$key] = $str;
    }
  }

  /**
   * Translate text id
   *
   * Uses all paramters after $id for sprintf()
   *
   * @param string $id
   * @return string
   * @uses getf
   */
  public static function get() {
    $args = func_get_args();
    $ustr = strtoupper($args[0]);

    list($NameSpace, $key) = explode(self::$NameSpaceSeparator, $ustr, 2);
    if (isset(self::$Translation[$NameSpace][$key])) {
      $args[0] = self::$Translation[$NameSpace][$key];
      return self::getf($args);
    } else
      return '[['.$ustr.']]';
  }

  /**
   * Translate text id, considers singular/plural
   * looks for ID_1, ID_N or ID as default
   *
   * Uses all paramters after $id for sprintf()
   *
   * @param int $n Number to check for singular/plural
   * @param string $id
   * @return string
   * @uses getf
   */
  public static function getN() {
    $args = func_get_args();
    $ustr = strtoupper($args[0]);

    // first argument after text id is interpreted as amount
    $n = isset($args[1]) ? (int)$args[1] : 0;

    list($NameSpace, $key) = explode(self::$NameSpaceSeparator, $ustr, 2);
    if (isset(self::$Translation[$NameSpace])) {
      $t =& self::$Translation[$NameSpace];
      switch (TRUE) {
        case ($n == 1 AND isset($t[$key.'_1'])):
          // singular
          $args[0] = $t[$key.'_1'];
          break;
        case isset($t[$key.'_N']):
          // plural
          $args[0] = $t[$key.'_N'];
          break;
        case isset($t[$key]):
          // default
          $args[0] = $t[$key];
          break;
        default:
          // not found in namespace
          return '[['.$ustr.']]';
      } // switch
      return self::getf($args);
    } else
      // namespace not found
      return '[['.$ustr.']]';
  }

  /**
   * Translate direct a text string
   *
   * Uses all paramters after $id for sprintf()
   *
   * @return string
   */
  public static function getf() {
    $args = func_get_args();
    // called internally?
    if (is_array($args[0])) $args = $args[0];
    $str = array_shift($args);
    $str = vsprintf($str, $args);
    $str = str_replace("\r", '', $str);
    $str = str_replace(array("\n", "\t"), ' ', $str);
    return $str;
  }

  /**
   * Translate text id with default value
   *
   * Uses all paramters after $id for sprintf()
   *
   * LAST PARAMETER have to be the default value.
   *
   * EXCEPT: If ONLY one paramter given, default value will be '' (empty string)!
   *
   * @return string
   * @uses getf
   */
  public static function getNvl() {
    // last paramter HAVE TO BE the default value!!
    $args = func_get_args();
    $nvl = (count($args) > 1) ? array_pop($args) : '';
    $ustr = strtoupper($args[0]);
    list($Ext, $key) = explode(self::$NameSpaceSeparator, $ustr, 2);
    $return = $nvl;
    if (isset(self::$Translation[$Ext][$key])) {
      $args[0] = self::$Translation[$Ext][$key];
      $return = self::getf($args);
    }
    return $return;
  }

  /**
   * Return all translations
   *
   * @return array
   */
  public static function getAll() {
    return self::$Translation;
  }

  //---------------------------------------------------------------------------
  // PRIVATE
  //---------------------------------------------------------------------------

  /**
   *
   */
  private static $Translation = array();

  /**
   *
   */
  private static $Masks = array(
    array( '<',    '>'    ),
    array( "\x01", "\x02" ),
  );
  
}
