<?php
/**
 * AppUpdate package
 *
 * @package    AppUpdate
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2006-2009 Knut Kohl
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 * @version    1.0.0
 */

/**
 * Updates files of an application
 *
 * @package    AppUpdate
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2006-2009 Knut Kohl
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 * @version    1.0.0
 */
class AppUpdate {

  /**
   * Updatable flag for files
   */
  const UPD = '%u';

  /**
   * ID to cache in session
   */
  const SESSIONID = '.AutoUpdate';

  /**
   * Class constructor
   *
   * Configure the class instance using this parameters
   *  - server         => Server to update from, e.g. http://example.com
   *  - update_file    => File name with update infos, e.g. application.upd
   *  - check_version  => Check actual app. against version, e.g. 1.0.0
   *  - cache_lifespan => Cache the update infos for xxx seconds
   *
   * @param array $Config Configuration data
   */
  public function __construct( $Config=array() ) {
    if (!is_array($Config))
      throw new Exception(__CLASS__.': Constructor parameter $Config must be an array!');
    $this->Config = array_merge($this->Config, $Config);
  }

  /**
   * Checks whatever there are updates for the software
   *
   * To perform additional checks, provide a callback function following
   * this definition:
   *
   * <code>
   *   /**
   *    * @param bool &$update Update given file
   *    * @param string $file File to update
   *    * @param string $version Available version
   *    * /
   *   function CheckUpdateCallback( &$update, $file, $version ) {
   *     // modify the update flag as you need
   *     // e.g. check against result of last update, saved by the callback
   *     // during $this->Update()
   *   }
   * </code>
   *
   * @param string $callback Callback function for additional tests
   * @throws AppUpdateException In case of error during getting update informations
   * @return void
   */
  public function CheckUpdates( $callback='' ) {
    $this->Application    = array();
    $this->UpdatableFiles = 0;
    $this->Files          = array();

    if (session_id() == '') session_start();

    if ($this->get('cache_lifespan') > 0 AND
        isset($_SESSION[self::SESSIONID]['time']) AND
        time() < $_SESSION[self::SESSIONID]['time']+$this->get('cache_lifespan')) {
      $this->Application    = $_SESSION[self::SESSIONID]['Application'];
      $this->UpdatableFiles = $_SESSION[self::SESSIONID]['UpdatableFiles'];
      $this->Files          = $_SESSION[self::SESSIONID]['Files'];
      return;
    }

    $file = $this->get('server') . $this->get('update_file');
    if (!$rh = @fopen($file, 'rb'))
      throw new AppUpdateException('Can\'t open '.$file, 1);

    // Use a temporary file
    $tempname = tempnam(ini_get('upload_tmp_dir'), 'upd.');
    if (!$wh = @fopen($tempname, 'wb'))
      throw new AppUpdateException('Can\'t open '.$tempname.' for writing.', 2);

    while (!feof($rh)) {
      if (fwrite($wh, fread($rh, 512)) === FALSE) {
        fclose($rh);
        fclose($wh);
        throw new AppUpdateException('Can\'t write to '.$tempname, 3);
      }
    }
    fclose($rh);
    fclose($wh);

    $this->Files = parse_ini_file($tempname, TRUE);
    @unlink($tempname);

    if (isset($this->Files['application'])) {
      $this->Application = array_merge($this->Application, $this->Files['application']);
      unset($this->Files['application']);
    }

    foreach ($this->Files as $file => $data) {
      if (!isset($data['version']) OR !isset($data['source']))
        throw new AppUpdateException('Format error in ['.$file.']', 4);
      if (!isset($data['comment'])) $this->Files[$file]['comment'] = '';
    }

    ksort($this->Files);

    foreach ($this->Files as $file => $data) {
      $update = version_compare($this->get('check_version'), $data['version'], '<');
      if ($callback)
        call_user_func_array($callback, array(&$update, $file, $data['version']));
      $this->Files[$file][self::UPD] = $update;
      if ($update) $this->UpdatableFiles++;
    }

    if (!$this->get('cache_lifespan')) return;

    $_SESSION[self::SESSIONID] = array(
      'time'           => time(),
      'Application'    => $this->Application,
      'UpdatableFiles' => $this->UpdatableFiles,
      'Files'          => $this->Files
    );
  }

  /**
   * Get actual application information
   *
   * @return array Array ( 'version' => version, 'comment' => comment )
   */
  public function getApplicationVersion() {
    return $this->Application;
  }

  /**
   * How many files are updatable
   *
   * @return int
   */
  public function getUpdatableCount() {
    return $this->UpdatableFiles;
  }

  /**
   * Return the list of all files available
   *
   * @param bool $all Return all files or only updatable
   * @return array Array of File => Version
   */
  public function getFiles( $all=TRUE ) {
    $return = array();
    foreach ($this->Files as $file=>$data) {
      if ($all OR $data[self::UPD]) {
        $return[$file] = $data;
      }
    }
    return $return;
  }

  /**
   * Checks if the file is writable or not.
   *
   * @param string $file File name
   * @return bool
   */
  public function isWritable( $file ) {
    // will work in despite of Windows ACLs bug
    // NOTE: use a trailing slash for folders!!!
    // see http://bugs.php.net/bug.php?id=27609
    // see http://bugs.php.net/bug.php?id=30931

    $exists = file_exists($file);
    if ($fh = @fopen($file, 'a')) {
      fclose($fh);
      if (!$exists) unlink($file);
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Updates the files with the ones from the URL
   *
   * Skip all, if at least one file ist not writable
   *
   * To perform additional actions, provide a callback function following
   * this definition:
   *
   * <code>
   *   /**
   *    * @param string $file Updated file
   *    * @param string $version New version
   *    * /
   *   function AfterUpdateCallback( $file, $version ) {
   *     // e.g. save the new version and use it during the next update run
   *     // via the callback in $this->CheckForUpdates() to skip up-to-date files
   *   }
   * </code>
   *
   * @param bool $progress
   * @param string $callback Callback function after update
   * @return bool
   */
  public function UpdateFiles( $callback='' ) {

    // extract updatable files and check if writable
    $files = array();
    foreach ($this->Files as $file=>$data) {
      if ($data[self::UPD]) {
        if (!$this->isWritable($file)) return FALSE;
        $files[$file] = $data;
      }
    }

    // all required wriable, let's go
    foreach ($files as $file=>$data) {

      $source = $this->get('server') . $data['source'];
      if (!$rh = @fopen($source ,'rb'))
        throw new AppUpdateException('Can\'t open '.$source, 11);

      if (!$wh = @fopen($file, 'wb'))
        throw new AppUpdateException('Can\'t open '.$file.' for writing.', 12);

      while (!feof($rh)) {
        if (fwrite($wh, fread($rh, 512)) === FALSE) {
          fclose($wh);
          throw new AppUpdateException('Can\'t write to '.$file, 13);
        }
      }
      fclose($rh);  fclose($wh);

      if ($callback)
        call_user_func_array($callback, array($file, $data['version']));
    }

    if (session_id() == '') session_start();
    unset($_SESSION[self::SESSIONID]);

    return TRUE;
  }

  // -------------------------------------------------------------------------
  // PROTECTED
  // -------------------------------------------------------------------------

  /**
   * Initial configuration
   */
  protected $Config = array(
    'server'         => '',
    'update_file'    => '',
    'check_version'  => '1',
    'cache_lifespan' => 300,
  );

  /**
   * Application version
   */
  protected $Application = array(
    'version' => '0',
    'comment' => '',
    'url'     => '',
  );

  /**
   * Count of updatable files
   */
  protected $UpdatableFiles;

  /**
   * Whole file list
   */
  protected $Files;

  /**
   *
   */
  protected function get( $key ) {
    return isset($this->Config[$key]) ? $this->Config[$key] : NULL;
  }

}

/**
 * @package AppUpdate
 * @author  Knut Kohl <knutkohl@users.sourceforge.net>
 * @version 1.0.0
 */
class AppUpdateException extends Exception {}

/*

$updater = new AppUpdate('http://update.es-f.com/', 'es-f.upd', ESF_VERSION);

try {
  $updater->CheckUpdates($cnt, 'callback_CheckFileVersion');
} catch (AppUpdateException $e) {
  Messages::addError($e->getMessage().' ('.$e->getCode().')');
}

if ($cnt) {
  Messages::addSuccess('There are '.$cnt.' files ready for update:');
  // get updatable files
  foreach ($updater->getFiles(FALSE) as $file=>$version) {
    Messages::addInfo($file.' => '.$version);
    // check if the file is writable
    if (!$updater->isWritable($file))
      Messages ::addError('ERROR: File is not writable.' );
  }
  try {
    $updater->Update('callback_SaveFileVersion');
  } catch (AppUpdateException $e) {
    Messages::addError($e->getMessage().' ('.$e->getCode().')');
  }
}

function callback_CheckFileVersion( &$update, $file, $version ) {
  if ($last = @file(callback_FileName($file)))
    $update = version_compare($last[1], $version, '<');
}

function callback_SaveFileVersion( $file, $version ) {
  file_put_contents(callback_FileName($file), $file."\n".$version);
}

function callback_FileName( $file ) {
  return ini_get('upload_tmp_dir').'/'.md5($file).'.version';
}


*/
